unit LibLoader;

{$M+}
interface

uses
  Windows, SysUtils, Classes;

type
  { evenement dclanch en cas d'erreur dans TLibraryLoader.GetProc/GetProcs }
  TLibGetProcErrorEvent = procedure (const aLibName, aProcName: string; const aErrorCode: LongWord) of object;

  { evenement dclanch en cas d'erreur dans TLibraryLoader.Load/Unload }
  TLibErrorEvent        = procedure (const aLibName: string; const aErrorCode: LongWord) of object;


  TLibraryLoader = class
  private
    fLibName        : string;                        // nom de la librairie  charger
    fLibHandle      : HMODULE;                       // Handle du module charg (LoadLibrary)
    fLoaded         : boolean;                       // true si la librairie a t charge correctement
    fCount             : integer;                       // compteur de procedures charges avec succs
    fOnError        : TLibErrorEvent;
    fOnGetProcError : TLibGetProcErrorEvent;

  protected
    procedure Error(const aErrorCode: LongWord); virtual;
    procedure ProcError(const aProcName: string; const aErrorCode: LongWord); virtual;

  published
    property OnLibError     : TLibErrorEvent        read fOnError        write fOnError;
    property OnGetProcError : TLibGetProcErrorEvent read fOnGetProcError write fOnGetProcError;
    property LibName        : string  read fLibName;
    property LibHandle      : HMODULE read fLibHandle;
    property Loaded         : boolean read fLoaded;
    property ProcCount      : integer read fCount;

  public

    { CREATE
        Cre un nouveau Loader

        aLibName     : nom de la librairie  charger (par exemple : Kernel32.dll)
        aListGetProc : facultatif, true si vous dsirez maintenir une liste des procedures charges avec succs
    }
    constructor Create(const aLibName: string);

    { LOAD
        Charge la librairie definie dans le constructeur
        renvois true si cette dernire  bien t charge, sinon false.
        plusieurs appels  Load ne recharge pas la librairie et ne provoque pas d'erreurs.
        appel l'evenement OnLibError en cas d'erreur.

        exemple :
          LibKernel32 := TLibraryLoader.Create('kernel32.dll');
    }
    function Load: boolean;

    { UNLOAD
        Decharge la librairie charge avec Load.
        renvois true si tout c'est bien droul, sinon false.
        appel l'evenement OnLibError en cas d'erreur.
    }
    function Unload: boolean;

    { GETPROC
        Charge une procedure de la librairie charge.
        renvois true si la procedure  t trouve, sinon false.
        appel l'evenement OnGetProcError en cas d'erreur.

        aProcName : nom de la procedure  charger
        vProc     : pointeur vers votre variable de type procedure ou function

        exemple :
          GetProc('GetTickCount', @Kernel32GetTickCount)
    }
    function GetProc(const aProcName: string; var vProc: pointer): boolean;

    { DESTROY
        n'appelez Destroy appelez la methode FREE.
    }
    destructor Destroy; override;
  end;


implementation


{ TLibrary }

constructor TLibraryLoader.Create(const aLibName: string);
begin
  fLibName   := aLibName;
  fLibHandle := INVALID_HANDLE_VALUE;
  fLoaded    := false;
  fCount        := 0;
end;

destructor TLibraryLoader.Destroy;
begin
  Unload;
  inherited;
end;

procedure TLibraryLoader.Error(const aErrorCode: LongWord);
begin
  if (aErrorCode <> 0) and assigned(fOnError) then
    fOnError(fLibName, aErrorCode);
end;


function TLibraryLoader.GetProc(const aProcName: string; var vProc: pointer): boolean;
begin
  result := false;
  vProc   := nil;
  if fLoaded then
  begin
    SetLastError(0);
    vProc  := GetProcAddress(fLibHandle, PChar(aProcName));
    ProcError(aProcName, GetLastError);
    result := vProc <> nil;
    if result then
      inc(fCount);
  end;
end;

function TLibraryLoader.Load: boolean;
begin
  if not fLoaded then
  begin
    SetLastError(0);
    fLibHandle := LoadLibrary(PChar(fLibName));
    Error(GetLastError);
    fLoaded    := fLibHandle <> INVALID_HANDLE_VALUE;
  end;
  result := fLoaded;
end;

procedure TLibraryLoader.ProcError(const aProcName: string; const aErrorCode: LongWord);
begin
  if (aErrorCode <> 0) and assigned(fOnGetProcError) then
    fOnGetProcError(fLibName, aProcName, aErrorCode);
end;

function TLibraryLoader.Unload: boolean;
begin
  if fLoaded then
  begin
    SetLastError(0);
    fLoaded := FreeLibrary(fLibHandle);
    Error(GetLastError);
  end;
  result  := fLoaded;
end;


end.
